@@ -20,6 +20,7 @@
 
 #include <Libkleo/Compat>
 #include <Libkleo/DirectoryServicesWidget>
+#include <Libkleo/GnuPG>
 #include <Libkleo/KeyserverConfig>
 
 #include <QGpgME/CryptoConfig>
@@ -102,15 +103,19 @@ private:
     Kleo::LabelledWidget<QSpinBox> mMaxItems;
     QCheckBox *mFetchMissingSignerKeysCB = nullptr;
     QCheckBox *mQueryWKDsForAllUserIDsCB = nullptr;
+    QCheckBox *mUseKeyServerCheckBox = nullptr;
+    QCheckBox *mRetrieveKeysCheckBox = nullptr;
 
     QGpgME::CryptoConfigEntry *mOpenPGPServiceEntry = nullptr;
     QGpgME::CryptoConfigEntry *mTimeoutConfigEntry = nullptr;
     QGpgME::CryptoConfigEntry *mMaxItemsConfigEntry = nullptr;
+    QGpgME::CryptoConfigEntry *mRetrieveKeysEntry = nullptr;
 
     QGpgME::CryptoConfig *mConfig = nullptr;
 };
 
-DirectoryServicesConfigurationPage::Private::Private(DirectoryServicesConfigurationPage *q)
+DirectoryServicesConfigurationPage::Private::Private(DirectoryServicesConfigurationPage *qq)
+    : q{qq}
 {
     mConfig = QGpgME::cryptoConfig();
     auto glay = new QGridLayout(q);
@@ -118,15 +123,59 @@ DirectoryServicesConfigurationPage::Private::Private(DirectoryServicesConfigurat
     // OpenPGP keyserver
     int row = 0;
     {
-        auto l = new QHBoxLayout{};
-        l->setContentsMargins(0, 0, 0, 0);
+        auto groupBox = new QGroupBox{i18nc("@title", "OpenPGP Keyserver")};
+        groupBox->setFlat(true);
+        auto groupBoxLayout = new QVBoxLayout{groupBox};
+        groupBoxLayout->setContentsMargins({});
 
-        mOpenPGPKeyserverEdit.createWidgets(q);
-        mOpenPGPKeyserverEdit.label()->setText(i18n("OpenPGP keyserver:"));
-        l->addWidget(mOpenPGPKeyserverEdit.label());
-        l->addWidget(mOpenPGPKeyserverEdit.widget());
+        groupBoxLayout->addWidget(new QLabel(i18n("Please note that only one OpenPGP keyserver can be configured.")));
 
-        glay->addLayout(l, row, 0, 1, 3);
+        {
+            mUseKeyServerCheckBox = new QCheckBox(i18nc("@label:checkbox", "Use OpenPGP keyserver"));
+            groupBoxLayout->addWidget(mUseKeyServerCheckBox);
+        }
+        {
+            auto l = new QHBoxLayout{};
+            l->setContentsMargins(0, 0, 0, 0);
+
+            groupBoxLayout->addLayout(l);
+
+            mOpenPGPKeyserverEdit.createWidgets(q);
+            mOpenPGPKeyserverEdit.label()->setText(i18n("OpenPGP keyserver:"));
+            if (engineIsVersion(2, 4, 4) //
+                || (engineIsVersion(2, 2, 42) && !engineIsVersion(2, 3, 0))) {
+                mOpenPGPKeyserverEdit.widget()->setToolTip( //
+                    xi18nc("@info:tooltip",
+                           "Enter the address of the keyserver to use when searching for OpenPGP certificates and "
+                           "when uploading OpenPGP certificates. If you do not enter an address then an internal "
+                           "default will be used. To disable the use of an OpenPGP keyserver enter the special value <emphasis>none</emphasis>."));
+            }
+            l->addWidget(mOpenPGPKeyserverEdit.label());
+            l->addWidget(mOpenPGPKeyserverEdit.widget());
+        }
+
+        connect(mUseKeyServerCheckBox, &QCheckBox::toggled, mOpenPGPKeyserverEdit.widget(), &QLineEdit::setEnabled);
+        connect(mUseKeyServerCheckBox, &QCheckBox::toggled, q, [this]() {
+            if (!mUseKeyServerCheckBox->isChecked()) {
+                mOpenPGPKeyserverEdit.widget()->setText(QStringLiteral("none"));
+                q->changed();
+            } else if (mOpenPGPKeyserverEdit.widget()->text() == QStringLiteral("none")) {
+                mOpenPGPKeyserverEdit.widget()->setText({});
+                q->changed();
+            }
+        });
+
+        {
+            mRetrieveKeysCheckBox = new QCheckBox(i18nc("@label:checkbox", "Search missing keys when verifying a signature"));
+            groupBoxLayout->addWidget(mRetrieveKeysCheckBox);
+            mRetrieveKeysEntry = configEntry("gpg", "auto-key-retrieve", CryptoConfigEntry::ArgType_None, SingleValue, DoNotShowError);
+            mRetrieveKeysCheckBox->setEnabled(!mRetrieveKeysEntry->isReadOnly());
+            connect(mRetrieveKeysCheckBox, &QCheckBox::toggled, q, [this]() {
+                q->changed();
+            });
+        }
+
+        glay->addWidget(groupBox, row, 0, 1, 3);
         connect(mOpenPGPKeyserverEdit.widget(), &QLineEdit::textEdited, q, &DirectoryServicesConfigurationPage::changed);
     }
 
@@ -232,6 +281,7 @@ void DirectoryServicesConfigurationPage::Private::load(const Kleo::Settings &set
 
             mDirectoryServices->setDisabled(true);
         }
+        mRetrieveKeysCheckBox->setChecked(mRetrieveKeysEntry->boolValue());
     }
 
     {
@@ -252,7 +302,9 @@ void DirectoryServicesConfigurationPage::Private::load(const Kleo::Settings &set
         }
 
         mOpenPGPKeyserverEdit.widget()->setText(mOpenPGPServiceEntry && mOpenPGPServiceEntry->isSet() ? mOpenPGPServiceEntry->stringValue() : QString());
-        mOpenPGPKeyserverEdit.setEnabled(mOpenPGPServiceEntry && !mOpenPGPServiceEntry->isReadOnly());
+        mUseKeyServerCheckBox->setChecked(mOpenPGPKeyserverEdit.widget()->text() != QStringLiteral("none"));
+        mUseKeyServerCheckBox->setEnabled(mOpenPGPServiceEntry && !mOpenPGPServiceEntry->isReadOnly());
+        mOpenPGPKeyserverEdit.setEnabled(mOpenPGPServiceEntry && !mOpenPGPServiceEntry->isReadOnly() && mUseKeyServerCheckBox->isChecked());
         if (newEntry && !newEntry->defaultValue().isNull()) {
             mOpenPGPKeyserverEdit.widget()->setPlaceholderText(newEntry->defaultValue().toString());
         } else {
@@ -366,10 +418,13 @@ void DirectoryServicesConfigurationPage::Private::save()
         const auto keyserver = mOpenPGPKeyserverEdit.widget()->text().trimmed();
         if (keyserver.isEmpty()) {
             mOpenPGPServiceEntry->resetToDefault();
+        } else if (keyserver == QLatin1StringView{"none"}) {
+            mOpenPGPServiceEntry->setStringValue(keyserver);
         } else {
-            const auto keyserverUrl = keyserver.contains(QLatin1String{"://"}) ? keyserver : (QLatin1String{"hkps://"} + keyserver);
+            const auto keyserverUrl = keyserver.contains(QLatin1StringView{"://"}) ? keyserver : (QLatin1String{"hkps://"} + keyserver);
             mOpenPGPServiceEntry->setStringValue(keyserverUrl);
         }
+        mRetrieveKeysEntry->setBoolValue(mRetrieveKeysCheckBox->isChecked());
     }
 
     const QTime time{mTimeout.widget()->time()};
@@ -401,6 +456,9 @@ void DirectoryServicesConfigurationPage::Private::defaults()
     if (mMaxItemsConfigEntry && !mMaxItemsConfigEntry->isReadOnly()) {
         mMaxItemsConfigEntry->resetToDefault();
     }
+    if (mRetrieveKeysEntry && !mRetrieveKeysEntry->isReadOnly()) {
+        mRetrieveKeysEntry->resetToDefault();
+    }
 
     Settings settings;
     settings.setRetrieveSignerKeysAfterImport(settings.findItem(QStringLiteral("RetrieveSignerKeysAfterImport"))->getDefault().toBool());
@@ -421,7 +479,7 @@ CryptoConfigEntry *DirectoryServicesConfigurationPage::Private::configEntry(cons
         if (showError == DoShowError) {
             KMessageBox::error(
                 q,
-                i18n("Backend error: gpgconf does not seem to know the entry for %1/%2", QLatin1String(componentName), QLatin1String(entryName)));
+                i18n("Backend error: gpgconf does not seem to know the entry for %1/%2", QLatin1StringView(componentName), QLatin1String(entryName)));
         }
         return nullptr;
     }
@@ -429,8 +487,8 @@ CryptoConfigEntry *DirectoryServicesConfigurationPage::Private::configEntry(cons
         if (showError == DoShowError) {
             KMessageBox::error(q,
                                i18n("Backend error: gpgconf has wrong type for %1/%2: %3 %4",
-                                    QLatin1String(componentName),
-                                    QLatin1String(entryName),
+                                    QLatin1StringView(componentName),
+                                    QLatin1StringView(entryName),
                                     entry->argType(),
                                     entry->isList()));
         }
