@@ -16,11 +16,12 @@
 
 #include <set>
 
-template <typename K, typename U> class QMap;
+template<typename K, typename U>
+class QMap;
 
 namespace Kleo
 {
-class CertificatePair;
+struct CertificatePair;
 
 namespace Crypto
 {
@@ -38,9 +39,9 @@ public:
 
     void setAllowedProtocols(const std::set<GpgME::Protocol> &allowedProtocols);
     void setSelectedCertificates(const CertificatePair &certificates);
-    Q_REQUIRED_RESULT CertificatePair selectedCertificates() const;
+    [[nodiscard]] CertificatePair selectedCertificates() const;
 
-    Q_REQUIRED_RESULT bool rememberAsDefault() const;
+    [[nodiscard]] bool rememberAsDefault() const;
 
 private:
     SigningCertificateSelectionWidget *const widget;
@@ -49,5 +50,3 @@ private:
 }
 }
 }
-
-
