@@ -11,9 +11,8 @@
 
 #include "certificateselectiondialog.h"
 
-#include "settings.h"
-
-#include "conf/groupsconfigdialog.h"
+#include <kleopatraapplication.h>
+#include <settings.h>
 
 #include <view/keytreeview.h>
 #include <view/searchbar.h>
@@ -26,6 +25,7 @@
 #include <Libkleo/KeyCache>
 #include <Libkleo/KeyGroup>
 #include <Libkleo/KeyListModel>
+#include <Libkleo/UserIDProxyModel>
 
 #include <commands/importcertificatefromfilecommand.h>
 #include <commands/lookupcertificatescommand.h>
@@ -110,14 +110,8 @@ private:
     }
     void manageGroups()
     {
-        KConfigDialog *dialog = KConfigDialog::exists(GroupsConfigDialog::dialogName());
-        if (dialog) {
-            // reparent the dialog to ensure it's shown on top of the modal CertificateSelectionDialog
-            dialog->setParent(q, Qt::Dialog);
-        } else {
-            dialog = new GroupsConfigDialog(q);
-        }
-        dialog->show();
+        // ensure that the dialog is shown on top of the modal CertificateSelectionDialog
+        KleopatraApplication::instance()->openOrRaiseGroupsConfigDialog(q);
     }
     void slotKeysMayHaveChanged();
     void slotCurrentViewChanged(QAbstractItemView *newView);
@@ -234,9 +228,8 @@ CertificateSelectionDialog::CertificateSelectionDialog(QWidget *parent)
     : QDialog(parent)
     , d(new Private(this))
 {
-    const KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kleopatracertificateselectiondialogrc"));
-    d->ui.tabWidget.loadViews(config.data());
-    const KConfigGroup geometry(config, QStringLiteral("Geometry"));
+    d->ui.tabWidget.loadViews(KSharedConfig::openStateConfig(), QStringLiteral("CertificateSelectionDialog"), TabWidget::ShowUserIDs);
+    const auto geometry = KSharedConfig::openStateConfig()->group(QStringLiteral("CertificateSelectionDialog"));
     resize(geometry.readEntry("size", size()));
     d->slotKeysMayHaveChanged();
 }
@@ -378,6 +371,21 @@ std::vector<Key> CertificateSelectionDialog::selectedCertificates() const
     return model->keys(getSelectedRows(d->ui.tabWidget.currentView()));
 }
 
+std::vector<UserID> CertificateSelectionDialog::selectedUserIDs() const
+{
+    const auto model = d->ui.tabWidget.currentView()->model();
+    Q_ASSERT(model);
+
+    std::vector<UserID> userIds;
+    for (const auto &index : getSelectedRows(d->ui.tabWidget.currentView())) {
+        const auto userID = model->data(index, KeyList::UserIDRole).value<UserID>();
+        if (!userID.isNull()) {
+            userIds.push_back(userID);
+        }
+    }
+    return userIds;
+}
+
 Key CertificateSelectionDialog::selectedCertificate() const
 {
     const std::vector<Key> keys = selectedCertificates();
@@ -393,9 +401,8 @@ std::vector<KeyGroup> CertificateSelectionDialog::selectedGroups() const
 
 void CertificateSelectionDialog::hideEvent(QHideEvent *e)
 {
-    KSharedConfig::Ptr config = KSharedConfig::openConfig(QStringLiteral("kleopatracertificateselectiondialogrc"));
-    d->ui.tabWidget.saveViews(config.data());
-    KConfigGroup geometry(config, QStringLiteral("Geometry"));
+    d->ui.tabWidget.saveViews();
+    auto geometry = KSharedConfig::openStateConfig()->group(QStringLiteral("CertificateSelectionDialog"));
     geometry.writeEntry("size", size());
     QDialog::hideEvent(e);
 }
