@@ -1,5 +1,5 @@
 /* -*- mode: c++; c-basic-offset:4 -*-
-    dialogs/revokekeydialog.h
+    dialogs/revokekeydialog.cpp
 
     This file is part of Kleopatra, the KDE keymanager
     SPDX-FileCopyrightText: 2022 g10 Code GmbH
@@ -181,7 +181,7 @@ private:
 
     bool descriptionHasAcceptableInput() const
     {
-        return !q->description().contains(QLatin1String{"\n\n"});
+        return !q->description().contains(QLatin1StringView{"\n\n"});
     }
 
     QString descriptionErrorMessage() const
@@ -234,7 +234,7 @@ private:
         // screen readers say something like "invalid entry" if this state is set;
         // emulate this by adding "invalid entry" to the accessible name of the input field
         // and its label
-        const auto name = errorShown ? descriptionAccessibleName + QLatin1String{", "} + invalidEntryText() //
+        const auto name = errorShown ? descriptionAccessibleName + QLatin1StringView{", "} + invalidEntryText() //
                                      : descriptionAccessibleName;
         if (ui.descriptionLabel->accessibleName() != name) {
             ui.descriptionLabel->setAccessibleName(name);
