@@ -201,23 +201,12 @@ void Card::setHasNullPin(bool value)
     mHasNullPin = value;
 }
 
-bool Card::canLearnKeys() const
-{
-    return mCanLearn;
-}
-
-void Card::setCanLearnKeys(bool value)
-{
-    mCanLearn = value;
-}
-
 bool Card::operator==(const Card &other) const
 {
-    return mCanLearn == other.mCanLearn && mHasNullPin == other.mHasNullPin && mStatus == other.mStatus && mSerialNumber == other.mSerialNumber
-        && mAppName == other.mAppName && mAppVersion == other.mAppVersion && mCardType == other.mCardType && mCardVersion == other.mCardVersion
-        && mCardHolder == other.mCardHolder && mSigningKeyRef == other.mSigningKeyRef && mEncryptionKeyRef == other.mEncryptionKeyRef
-        && mAuthenticationKeyRef == other.mAuthenticationKeyRef && mPinStates == other.mPinStates && mErrMsg == other.mErrMsg && mKeyInfos == other.mKeyInfos
-        && mCardInfo == other.mCardInfo;
+    return mHasNullPin == other.mHasNullPin && mStatus == other.mStatus && mSerialNumber == other.mSerialNumber && mAppName == other.mAppName
+        && mAppVersion == other.mAppVersion && mCardType == other.mCardType && mCardVersion == other.mCardVersion && mCardHolder == other.mCardHolder
+        && mSigningKeyRef == other.mSigningKeyRef && mEncryptionKeyRef == other.mEncryptionKeyRef && mAuthenticationKeyRef == other.mAuthenticationKeyRef
+        && mPinStates == other.mPinStates && mErrMsg == other.mErrMsg && mKeyInfos == other.mKeyInfos && mCardInfo == other.mCardInfo;
 }
 
 bool Card::operator!=(const Card &other) const
@@ -263,7 +252,6 @@ void Card::setCardInfo(const std::vector<std::pair<std::string, std::string>> &i
         qCDebug(KLEOPATRA_LOG) << pair.first.c_str() << ":" << pair.second.c_str();
         parseCardInfo(pair.first, pair.second);
     }
-    processCardInfo();
 }
 
 namespace
@@ -330,10 +318,6 @@ void Card::parseCardInfo(const std::string &name, const std::string &value)
     }
 }
 
-void Card::processCardInfo()
-{
-}
-
 void Card::addCardInfo(const std::string &name, const std::string &value)
 {
     mCardInfo.insert({name, value});
@@ -360,3 +344,13 @@ std::string Card::keyFingerprint(const std::string &keyRef) const
 {
     return cardInfo("KLEO-FPR-" + keyRef);
 }
+
+void Card::setDisplayAppName(const QString &displayAppName)
+{
+    mDisplayAppName = displayAppName;
+}
+
+QString Card::displayAppName() const
+{
+    return mDisplayAppName;
+}
