@@ -67,6 +67,8 @@ public:
 
     void setName(const QString &name);
     QString name() const;
+    void setComment(const QString &comment);
+    QString comment() const;
     void setDN(const QString &dn);
     QString dn() const;
     void setEmail(const QString &email);
@@ -77,6 +79,9 @@ public:
     void addURI(const QString &uri);
     std::vector<QString> uris() const;
 
+    void addDesignatedRevoker(const QString &fpr);
+    std::vector<QString> designatedRevokers() const;
+
     QString toString() const;
 
 private:
