@@ -14,14 +14,21 @@
 namespace kdtools
 {
 
-template <typename T>
+template<typename T>
 class pimpl_ptr
 {
     KDAB_DISABLE_COPY(pimpl_ptr);
     T *d;
+
 public:
-    pimpl_ptr() : d(new T) {}
-    explicit pimpl_ptr(T *t) : d(t) {}
+    pimpl_ptr()
+        : d(new T)
+    {
+    }
+    explicit pimpl_ptr(T *t)
+        : d(t)
+    {
+    }
     ~pimpl_ptr()
     {
         delete d;
@@ -61,11 +68,9 @@ public:
 // these are not implemented, so's we can catch their use at
 // link-time. Leaving them undeclared would open up a comparison
 // via operator unspecified-bool-type().
-template <typename T, typename S>
+template<typename T, typename S>
 void operator==(const pimpl_ptr<T> &, const pimpl_ptr<S> &);
-template <typename T, typename S>
+template<typename T, typename S>
 void operator!=(const pimpl_ptr<T> &, const pimpl_ptr<S> &);
 
 } // namespace kdtools
-
-
