@@ -16,8 +16,8 @@
 #include <Libkleo/KeyListModel>
 #include <Libkleo/KeyListSortFilterProxyModel>
 #include <Libkleo/KeyRearrangeColumnsProxyModel>
-#include <Libkleo/NavigatableTreeView>
 #include <Libkleo/Predicates>
+#include <Libkleo/TreeView>
 
 #include "utils/headerview.h"
 #include "utils/tags.h"
@@ -43,7 +43,7 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 
-#define TAGS_COLUMN 13
+static int tagsColumn;
 
 using namespace Kleo;
 using namespace GpgME;
@@ -53,13 +53,17 @@ Q_DECLARE_METATYPE(GpgME::Key)
 namespace
 {
 
-class TreeView : public NavigatableTreeView
+class TreeViewInternal : public Kleo::TreeView
 {
 public:
-    explicit TreeView(QWidget *parent = nullptr)
-        : NavigatableTreeView{parent}
+    explicit TreeViewInternal(QWidget *parent = nullptr)
+        : Kleo::TreeView{parent}
     {
-        header()->installEventFilter(this);
+        connect(this, &TreeView::columnEnabled, this, [this](int column) {
+            if (column == tagsColumn) {
+                Tags::enableTags();
+            }
+        });
     }
 
     QSize minimumSizeHint() const override
@@ -69,57 +73,11 @@ public:
     }
 
 protected:
-    bool eventFilter(QObject *watched, QEvent *event) override
-    {
-        Q_UNUSED(watched)
-        if (event->type() == QEvent::ContextMenu) {
-            auto e = static_cast<QContextMenuEvent *>(event);
-
-            if (!mHeaderPopup) {
-                mHeaderPopup = new QMenu(this);
-                mHeaderPopup->setTitle(i18n("View Columns"));
-                for (int i = 0; i < model()->columnCount(); ++i) {
-                    QAction *tmp = mHeaderPopup->addAction(model()->headerData(i, Qt::Horizontal).toString());
-                    tmp->setData(QVariant(i));
-                    tmp->setCheckable(true);
-                    mColumnActions << tmp;
-                }
-
-                connect(mHeaderPopup, &QMenu::triggered, this, [this](QAction *action) {
-                    const int col = action->data().toInt();
-                    if ((col == TAGS_COLUMN) && action->isChecked()) {
-                        Tags::enableTags();
-                    }
-                    if (action->isChecked()) {
-                        showColumn(col);
-                    } else {
-                        hideColumn(col);
-                    }
-
-                    auto tv = qobject_cast<KeyTreeView *>(parent());
-                    if (tv) {
-                        tv->resizeColumns();
-                    }
-                });
-            }
-
-            for (QAction *action : std::as_const(mColumnActions)) {
-                const int column = action->data().toInt();
-                action->setChecked(!isColumnHidden(column));
-            }
-
-            mHeaderPopup->popup(mapToGlobal(e->pos()));
-            return true;
-        }
-
-        return false;
-    }
-
     void focusInEvent(QFocusEvent *event) override
     {
         QTreeView::focusInEvent(event);
         // queue the invokation, so that it happens after the widget itself got focus
-        QMetaObject::invokeMethod(this, &TreeView::forceAccessibleFocusEventForCurrentItem, Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &TreeViewInternal::forceAccessibleFocusEventForCurrentItem, Qt::QueuedConnection);
     }
 
 private:
@@ -152,7 +110,7 @@ KeyTreeView::KeyTreeView(QWidget *parent)
     : QWidget(parent)
     , m_proxy(new KeyListSortFilterProxyModel(this))
     , m_additionalProxy(nullptr)
-    , m_view(new TreeView(this))
+    , m_view(new TreeViewInternal(this))
     , m_flatModel(nullptr)
     , m_hierarchicalModel(nullptr)
     , m_stringFilter()
@@ -166,7 +124,7 @@ KeyTreeView::KeyTreeView(const KeyTreeView &other)
     : QWidget(nullptr)
     , m_proxy(new KeyListSortFilterProxyModel(this))
     , m_additionalProxy(other.m_additionalProxy ? other.m_additionalProxy->clone() : nullptr)
-    , m_view(new TreeView(this))
+    , m_view(new TreeViewInternal(this))
     , m_flatModel(other.m_flatModel)
     , m_hierarchicalModel(other.m_hierarchicalModel)
     , m_stringFilter(other.m_stringFilter)
@@ -187,7 +145,7 @@ KeyTreeView::KeyTreeView(const QString &text,
     : QWidget(parent)
     , m_proxy(new KeyListSortFilterProxyModel(this))
     , m_additionalProxy(proxy)
-    , m_view(new TreeView(this))
+    , m_view(new TreeViewInternal(this))
     , m_flatModel(nullptr)
     , m_hierarchicalModel(nullptr)
     , m_stringFilter(text)
@@ -244,6 +202,28 @@ std::vector<int> KeyTreeView::columnSizes() const
     }
 }
 
+void KeyTreeView::restoreLayout(const KConfigGroup &group)
+{
+    if (!group.isValid() || !m_view->restoreColumnLayout(group.name())) {
+        // if config is empty then use default settings
+        // The numbers have to be in line with the order in
+        // setsSourceColumns above
+        m_view->hideColumn(5);
+
+        for (int i = 7; i < m_view->model()->columnCount(); ++i) {
+            m_view->hideColumn(i);
+        }
+        if (KeyCache::instance()->initialized()) {
+            QTimer::singleShot(0, this, &KeyTreeView::initializeColumnSizes);
+        }
+    } else {
+        m_onceResized = true;
+    }
+    if (!m_view->isColumnHidden(tagsColumn)) {
+        Tags::enableTags();
+    }
+}
+
 void KeyTreeView::init()
 {
     KDAB_SET_OBJECT_NAME(m_proxy);
@@ -298,21 +278,26 @@ void KeyTreeView::init()
 
     auto rearangingModel = new KeyRearrangeColumnsProxyModel(this);
     rearangingModel->setSourceModel(m_proxy);
-    rearangingModel->setSourceColumns(QList<int>() << KeyList::PrettyName //
-                                                     << KeyList::PrettyEMail //
-                                                     << KeyList::Validity //
-                                                     << KeyList::ValidFrom //
-                                                     << KeyList::ValidUntil //
-                                                     << KeyList::TechnicalDetails //
-                                                     << KeyList::KeyID //
-                                                     << KeyList::Fingerprint //
-                                                     << KeyList::OwnerTrust //
-                                                     << KeyList::Origin //
-                                                     << KeyList::LastUpdate //
-                                                     << KeyList::Issuer //
-                                                     << KeyList::SerialNumber //
-                                                     // If a column is added before this TAGS_COLUMN define has to be updated accordingly
-                                                     << KeyList::Remarks);
+    QList<int> columns = {
+        KeyList::PrettyName,
+        KeyList::PrettyEMail,
+        KeyList::Validity,
+        KeyList::ValidFrom,
+        KeyList::ValidUntil,
+        KeyList::TechnicalDetails,
+        KeyList::KeyID,
+        KeyList::Fingerprint,
+        KeyList::OwnerTrust,
+        KeyList::Origin,
+        KeyList::LastUpdate,
+        KeyList::Issuer,
+        KeyList::SerialNumber,
+        KeyList::Remarks,
+        KeyList::Algorithm,
+        KeyList::Keygrip,
+    };
+    tagsColumn = columns.indexOf(KeyList::Remarks);
+    rearangingModel->setSourceColumns(columns);
     m_view->setModel(rearangingModel);
 
     /* Handle expansion state */
@@ -354,11 +339,6 @@ void KeyTreeView::init()
     });
 
     updateModelConnections(nullptr, model());
-
-    resizeColumns();
-    if (m_group.isValid()) {
-        restoreLayout(m_group);
-    }
 }
 
 void KeyTreeView::restoreExpandState()
@@ -400,92 +380,7 @@ void KeyTreeView::setUpTagKeys()
     }
 }
 
-void KeyTreeView::saveLayout(KConfigGroup &group)
-{
-    QHeaderView *header = m_view->header();
-
-    QVariantList columnVisibility;
-    QVariantList columnOrder;
-    QVariantList columnWidths;
-    const int headerCount = header->count();
-    columnVisibility.reserve(headerCount);
-    columnWidths.reserve(headerCount);
-    columnOrder.reserve(headerCount);
-    for (int i = 0; i < headerCount; ++i) {
-        columnVisibility << QVariant(!m_view->isColumnHidden(i));
-        columnWidths << QVariant(header->sectionSize(i));
-        columnOrder << QVariant(header->visualIndex(i));
-    }
-
-    group.writeEntry("ColumnVisibility", columnVisibility);
-    group.writeEntry("ColumnOrder", columnOrder);
-    group.writeEntry("ColumnWidths", columnWidths);
-
-    group.writeEntry("SortAscending", (int)header->sortIndicatorOrder());
-    if (header->isSortIndicatorShown()) {
-        group.writeEntry("SortColumn", header->sortIndicatorSection());
-    } else {
-        group.writeEntry("SortColumn", -1);
-    }
-}
-
-void KeyTreeView::restoreLayout(const KConfigGroup &group)
-{
-    QHeaderView *header = m_view->header();
-
-    QVariantList columnVisibility = group.readEntry("ColumnVisibility", QVariantList());
-    QVariantList columnOrder = group.readEntry("ColumnOrder", QVariantList());
-    QVariantList columnWidths = group.readEntry("ColumnWidths", QVariantList());
-
-    if (columnVisibility.isEmpty()) {
-        // if config is empty then use default settings
-        // The numbers have to be in line with the order in
-        // setsSourceColumns above
-        m_view->hideColumn(5);
-
-        for (int i = 7; i < m_view->model()->columnCount(); ++i) {
-            m_view->hideColumn(i);
-        }
-        if (KeyCache::instance()->initialized()) {
-            QTimer::singleShot(0, this, &KeyTreeView::resizeColumns);
-        }
-    } else {
-        for (int i = 0; i < header->count(); ++i) {
-            if (i >= columnOrder.size() || i >= columnWidths.size() || i >= columnVisibility.size()) {
-                // An additional column that was not around last time we saved.
-                // We default to hidden.
-                m_view->hideColumn(i);
-                continue;
-            }
-            bool visible = columnVisibility[i].toBool();
-            int width = columnWidths[i].toInt();
-            int order = columnOrder[i].toInt();
-
-            header->resizeSection(i, width ? width : 100);
-            header->moveSection(header->visualIndex(i), order);
-            if ((i == TAGS_COLUMN) && visible) {
-                Tags::enableTags();
-            }
-            if (!visible) {
-                m_view->hideColumn(i);
-            }
-        }
-        m_onceResized = true;
-    }
-
-    int sortOrder = group.readEntry("SortAscending", (int)Qt::AscendingOrder);
-    int sortColumn = group.readEntry("SortColumn", 0);
-    if (sortColumn >= 0) {
-        m_view->sortByColumn(sortColumn, (Qt::SortOrder)sortOrder);
-    }
-}
-
-KeyTreeView::~KeyTreeView()
-{
-    if (m_group.isValid()) {
-        saveLayout(m_group);
-    }
-}
+KeyTreeView::~KeyTreeView() = default;
 
 static QAbstractProxyModel *find_last_proxy(QAbstractProxyModel *pm)
 {
@@ -630,17 +525,20 @@ void KeyTreeView::setHierarchicalView(bool on)
     Q_EMIT hierarchicalChanged(on);
 }
 
-void KeyTreeView::setKeys(const std::vector<Key> &keys)
+void KeyTreeView::setKeys(const std::vector<Key> &keys, const std::vector<Key::Origin> &extraOrigins)
 {
     std::vector<Key> sorted = keys;
-    _detail::sort_by_fpr(sorted);
-    _detail::remove_duplicates_by_fpr(sorted);
+
+    if (extraOrigins.empty()) {
+        _detail::sort_by_fpr(sorted);
+        _detail::remove_duplicates_by_fpr(sorted);
+    }
     m_keys = sorted;
     if (m_flatModel) {
-        m_flatModel->setKeys(sorted);
+        m_flatModel->setKeys(sorted, extraOrigins);
     }
     if (m_hierarchicalModel) {
-        m_hierarchicalModel->setKeys(sorted);
+        m_hierarchicalModel->setKeys(sorted, extraOrigins);
     }
 }
 
@@ -729,8 +627,12 @@ bool KeyTreeView::connectSearchBar(const SearchBar *bar)
     });
 }
 
-void KeyTreeView::resizeColumns()
+void KeyTreeView::initializeColumnSizes()
 {
+    if (m_onceResized || m_view->model()->rowCount() == 0) {
+        return;
+    }
+    m_onceResized = true;
     m_view->setColumnWidth(KeyList::PrettyName, 260);
     m_view->setColumnWidth(KeyList::PrettyEMail, 260);
 
@@ -759,10 +661,7 @@ void KeyTreeView::restoreStateAfterModelChange()
     }
 
     setUpTagKeys();
-    if (!m_onceResized) {
-        m_onceResized = true;
-        resizeColumns();
-    }
+    initializeColumnSizes();
 }
 
 #include "moc_keytreeview.cpp"
