@@ -7,10 +7,15 @@
 */
 #pragma once
 
+#include <Libkleo/Predicates>
+
 #include <QWidget>
+
 #include <gpgme++/error.h>
 
+#include <set>
 #include <string>
+#include <vector>
 
 class QLabel;
 class QPushButton;
@@ -19,14 +24,14 @@ class QScrollArea;
 namespace Kleo
 {
 class NullPinWidget;
-class KeyTreeView;
+class CardKeysView;
 
 namespace SmartCard
 {
 class NetKeyCard;
-} // namespace SmartCard
+}
 
-class NetKeyWidget: public QWidget
+class NetKeyWidget : public QWidget
 {
     Q_OBJECT
 public:
@@ -42,18 +47,16 @@ private:
 
 private:
     std::string mSerialNumber;
-    QLabel *mSerialNumberLabel = nullptr,
-           *mVersionLabel = nullptr,
-           *mLearnKeysLabel = nullptr,
-           *mErrorLabel = nullptr;
+
+    QLabel *mSerialNumberLabel = nullptr;
+    QLabel *mVersionLabel = nullptr;
+    QLabel *mErrorLabel = nullptr;
     NullPinWidget *mNullPinWidget = nullptr;
-    QPushButton *mLearnKeysBtn = nullptr,
-                *mKeyForCardKeysButton = nullptr,
-                *mCreateCSRButton = nullptr,
-                *mChangeNKSPINBtn = nullptr,
-                *mChangeSigGPINBtn = nullptr;
-    KeyTreeView *mTreeView = nullptr;
+    QPushButton *mKeyForCardKeysButton = nullptr;
+    QPushButton *mCreateCSRButton = nullptr;
+    QPushButton *mChangeNKSPINBtn = nullptr;
+    QPushButton *mChangeSigGPINBtn = nullptr;
+    CardKeysView *mCardKeysView = nullptr;
     QScrollArea *mArea = nullptr;
 };
 } // namespace Kleo
-
