@@ -24,8 +24,11 @@
 #include <Libkleo/KeyListModel>
 #include <Libkleo/KeyListSortFilterProxyModel>
 #include <Libkleo/Stl_Util>
+#include <Libkleo/UserIDProxyModel>
 
 #include <gpgme++/key.h>
+// needed for GPGME_VERSION_NUMBER
+#include <gpgme.h>
 
 #include <KActionCollection>
 #include <KConfig>
@@ -37,8 +40,10 @@
 #include <QTabWidget>
 
 #include <QAbstractProxyModel>
+#include <QHeaderView>
 #include <QMenu>
 #include <QRegularExpression>
+#include <QTimer>
 #include <QToolButton>
 #include <QTreeView>
 #include <QVBoxLayout>
@@ -110,8 +115,6 @@ public:
         return m_canChangeHierarchical;
     }
 
-    void saveTo(KConfigGroup &group) const;
-
     Page *clone() const override
     {
         return new Page(*this);
@@ -122,6 +125,17 @@ public:
         m_canBeClosed = m_canBeRenamed = m_canChangeStringFilter = m_canChangeKeyFilter = m_canChangeHierarchical = true;
     }
 
+    void closePage()
+    {
+        m_configGroup.deleteGroup();
+        m_configGroup.sync();
+    }
+
+    KConfigGroup configGroup() const
+    {
+        return m_configGroup;
+    }
+
 Q_SIGNALS:
     void titleChanged(const QString &title);
 
@@ -137,6 +151,7 @@ private:
     bool m_canChangeStringFilter : 1;
     bool m_canChangeKeyFilter : 1;
     bool m_canChangeHierarchical : 1;
+    KConfigGroup m_configGroup;
 };
 } // anon namespace
 
@@ -150,6 +165,7 @@ Page::Page(const Page &other)
     , m_canChangeStringFilter(other.m_canChangeStringFilter)
     , m_canChangeKeyFilter(other.m_canChangeKeyFilter)
     , m_canChangeHierarchical(other.m_canChangeHierarchical)
+    , m_configGroup(other.configGroup().config()->group(QUuid::createUuid().toString()))
 {
     init();
 }
@@ -170,6 +186,7 @@ Page::Page(const QString &title,
     , m_canChangeStringFilter(true)
     , m_canChangeKeyFilter(true)
     , m_canChangeHierarchical(true)
+    , m_configGroup(group)
 {
     init();
 }
@@ -192,6 +209,7 @@ Page::Page(const KConfigGroup &group, QWidget *parent)
     , m_canChangeStringFilter(!group.isEntryImmutable(STRING_FILTER_ENTRY))
     , m_canChangeKeyFilter(!group.isEntryImmutable(KEY_FILTER_ENTRY))
     , m_canChangeHierarchical(!group.isEntryImmutable(HIERARCHICAL_VIEW_ENTRY))
+    , m_configGroup(group)
 {
     init();
     setHierarchicalView(group.readEntry(HIERARCHICAL_VIEW_ENTRY, true));
@@ -205,33 +223,24 @@ Page::Page(const KConfigGroup &group, QWidget *parent)
 
 void Page::init()
 {
+#if GPGME_VERSION_NUMBER >= 0x011800 // 1.24.0
+    view()->setDragDropMode(QAbstractItemView::DragOnly);
+    view()->setDragEnabled(true);
+#endif
 }
 
 Page::~Page()
 {
 }
 
-void Page::saveTo(KConfigGroup &group) const
-{
-    group.writeEntry(TITLE_ENTRY, m_title);
-    group.writeEntry(STRING_FILTER_ENTRY, stringFilter());
-    group.writeEntry(KEY_FILTER_ENTRY, keyFilter() ? keyFilter()->id() : QString());
-    group.writeEntry(HIERARCHICAL_VIEW_ENTRY, isHierarchicalView());
-    QList<int> settings;
-    const auto sizes = columnSizes();
-    settings.reserve(sizes.size());
-    std::copy(sizes.cbegin(), sizes.cend(), std::back_inserter(settings));
-    group.writeEntry(COLUMN_SIZES, settings);
-    group.writeEntry(SORT_COLUMN, sortColumn());
-    group.writeEntry(SORT_DESCENDING, sortOrder() == Qt::DescendingOrder);
-}
-
 void Page::setStringFilter(const QString &filter)
 {
     if (!m_canChangeStringFilter) {
         return;
     }
     KeyTreeView::setStringFilter(filter);
+    m_configGroup.writeEntry(STRING_FILTER_ENTRY, stringFilter());
+    m_configGroup.sync();
 }
 
 void Page::setKeyFilter(const std::shared_ptr<KeyFilter> &filter)
@@ -245,6 +254,8 @@ void Page::setKeyFilter(const std::shared_ptr<KeyFilter> &filter)
     if (oldTitle != newTitle) {
         Q_EMIT titleChanged(newTitle);
     }
+    m_configGroup.writeEntry(KEY_FILTER_ENTRY, keyFilter() ? keyFilter()->id() : QString());
+    m_configGroup.sync();
 }
 
 void Page::setTitle(const QString &t)
@@ -260,6 +271,8 @@ void Page::setTitle(const QString &t)
     const QString newTitle = title();
     if (oldTitle != newTitle) {
         Q_EMIT titleChanged(newTitle);
+        m_configGroup.writeEntry(TITLE_ENTRY, m_title);
+        m_configGroup.sync();
     }
 }
 
@@ -287,6 +300,8 @@ void Page::setHierarchicalView(bool on)
         return;
     }
     KeyTreeView::setHierarchicalView(on);
+    m_configGroup.writeEntry(HIERARCHICAL_VIEW_ENTRY, on);
+    m_configGroup.sync();
 }
 
 void Page::setTemporary(bool on)
@@ -466,6 +481,8 @@ private:
     Actions currentPageActions;
     Actions otherPageActions;
     bool actionsCreated = false;
+    KSharedConfig::Ptr config;
+    QString configKey;
 };
 
 TabWidget::Private::Private(TabWidget *qq)
@@ -495,6 +512,9 @@ TabWidget::Private::Private(TabWidget *qq)
     connect(tabWidget->tabBar(), &QWidget::customContextMenuRequested, q, [this](const QPoint &p) {
         slotContextMenu(p);
     });
+    connect(tabWidget->tabBar(), &QTabBar::tabMoved, q, [this]() {
+        q->saveViews();
+    });
 }
 
 void TabWidget::Private::slotContextMenu(const QPoint &p)
@@ -616,10 +636,13 @@ void TabWidget::Private::slotPageHierarchyChanged(bool)
 
 void TabWidget::Private::slotNewTab()
 {
-    const KConfigGroup group = KSharedConfig::openConfig()->group(QString::asprintf("View #%u", tabWidget->count()));
+    auto group = KSharedConfig::openStateConfig()->group(QStringLiteral("%1:View %2").arg(configKey, QUuid::createUuid().toString()));
     Page *page = new Page(QString(), QStringLiteral("all-certificates"), QString(), nullptr, QString(), nullptr, group);
+    group.writeEntry(KEY_FILTER_ENTRY, QStringLiteral("all-certificates"));
+    group.sync();
     addView(page, currentPage());
     tabWidget->setCurrentIndex(tabWidget->count() - 1);
+    q->saveViews();
 }
 
 void TabWidget::Private::renamePage(Page *page)
@@ -652,7 +675,9 @@ void TabWidget::Private::closePage(Page *page)
         return;
     }
     Q_EMIT q->viewAboutToBeRemoved(page->view());
+    page->closePage();
     tabWidget->removeTab(tabWidget->indexOf(page));
+    q->saveViews();
     enableDisableCurrentPageActions();
 }
 
@@ -714,7 +739,7 @@ TabWidget::TabWidget(QWidget *p, Qt::WindowFlags f)
 
 TabWidget::~TabWidget()
 {
-    saveViews(KSharedConfig::openConfig().data());
+    saveViews();
 }
 
 void TabWidget::setFlatModel(AbstractKeyListModel *model)
@@ -956,7 +981,7 @@ void TabWidget::createActions(KActionCollection *coll)
         // but do not add those actions to the action collection
         auto action = new QAction(ad.text, coll);
         if (ad.icon) {
-            action->setIcon(QIcon::fromTheme(QLatin1String(ad.icon)));
+            action->setIcon(QIcon::fromTheme(QLatin1StringView(ad.icon)));
         }
         action->setEnabled(ad.actionState == Enabled);
         d->otherPageActions.insert(ad.name, action);
@@ -975,14 +1000,34 @@ void TabWidget::createActions(KActionCollection *coll)
 
 QAbstractItemView *TabWidget::addView(const QString &title, const QString &id, const QString &text)
 {
-    const KConfigGroup group = KSharedConfig::openConfig()->group(QString::asprintf("View #%u", d->tabWidget->count()));
+    auto group = KSharedConfig::openStateConfig()->group(QStringLiteral("%1:View %2").arg(d->configKey, QUuid::createUuid().toString()));
     Page *page = new Page(title, id, text, nullptr, QString(), nullptr, group);
+    group.writeEntry(KEY_FILTER_ENTRY, id);
+    group.sync();
     return d->addView(page, d->currentPage());
 }
 
-QAbstractItemView *TabWidget::addView(const KConfigGroup &group)
+QAbstractItemView *TabWidget::addView(const KConfigGroup &group, Options options)
 {
-    return d->addView(new Page(group), nullptr);
+    Page *page = nullptr;
+    if (options & ShowUserIDs) {
+        page = new Page(group.readEntry(TITLE_ENTRY),
+                        group.readEntry(KEY_FILTER_ENTRY),
+                        group.readEntry(STRING_FILTER_ENTRY),
+                        new UserIDProxyModel(this),
+                        {},
+                        nullptr,
+                        group);
+    } else {
+        page = new Page(group);
+    }
+    QMetaObject::invokeMethod(
+        this,
+        [page, group]() {
+            page->restoreLayout(group);
+        },
+        Qt::QueuedConnection);
+    return d->addView(page, nullptr);
 }
 
 QAbstractItemView *TabWidget::addTemporaryView(const QString &title, AbstractKeyListSortFilterProxyModel *proxy, const QString &tabToolTip)
@@ -1023,8 +1068,22 @@ QTreeView *TabWidget::Private::addView(Page *page, Page *columnReference)
     });
 
     if (columnReference) {
-        page->setColumnSizes(columnReference->columnSizes());
-        page->setSortColumn(columnReference->sortColumn(), columnReference->sortOrder());
+        QMetaObject::invokeMethod(
+            q,
+            [=]() {
+                page->setColumnSizes(columnReference->columnSizes());
+                page->setSortColumn(columnReference->sortColumn(), columnReference->sortOrder());
+                page->view()->saveColumnLayout(page->configGroup().name());
+            },
+            Qt::QueuedConnection);
+        for (auto i = 0; i < columnReference->view()->model()->columnCount(); i++) {
+            page->view()->setColumnHidden(i, columnReference->view()->isColumnHidden(i));
+            page->view()->header()->moveSection(page->view()->header()->visualIndex(i), columnReference->view()->header()->visualIndex(i));
+        }
+        for (auto row = 0; row < page->view()->model()->rowCount(); row++) {
+            page->view()->setExpanded(page->view()->model()->index(row, 0),
+                                      columnReference->view()->isExpanded(columnReference->view()->model()->index(row, 0)));
+        }
     }
 
     QAbstractItemView *const previous = q->currentView();
@@ -1042,54 +1101,41 @@ QTreeView *TabWidget::Private::addView(Page *page, Page *columnReference)
     return view;
 }
 
-static QStringList extractViewGroups(const KConfig *config)
+static QStringList extractViewGroups(const KConfigGroup &config)
 {
-    return config ? config->groupList().filter(QRegularExpression(QStringLiteral("^View #\\d+$"))) : QStringList();
+    return config.readEntry("Tabs", QStringList());
 }
 
-// work around deleteGroup() not deleting groups out of groupList():
-static const bool KCONFIG_DELETEGROUP_BROKEN = true;
-
-void TabWidget::loadViews(const KConfig *config)
+void TabWidget::loadViews(const KSharedConfig::Ptr &config, const QString &configKey, Options options)
 {
-    if (config) {
-        QStringList groupList = extractViewGroups(config);
-        groupList.sort();
-        for (const QString &group : std::as_const(groupList)) {
-            const KConfigGroup kcg(config, group);
-            if (!KCONFIG_DELETEGROUP_BROKEN || kcg.readEntry("magic", 0U) == 0xFA1AFE1U) {
-                addView(kcg);
-            }
-        }
+    d->config = config;
+    d->configKey = configKey;
+    QStringList groupList = extractViewGroups(config->group(configKey));
+    for (const QString &view : std::as_const(groupList)) {
+        addView(KConfigGroup(config, view), options);
     }
     if (!count()) {
         // add default view:
-        addView(i18n("All Certificates"), QStringLiteral("all-certificates"));
+        addView({}, QStringLiteral("all-certificates"));
     }
 }
 
-void TabWidget::saveViews(KConfig *config) const
+void TabWidget::saveViews()
 {
-    if (!config) {
+    if (!d->config) {
         return;
     }
-    const auto extraView{extractViewGroups(config)};
-    for (const QString &group : extraView) {
-        config->deleteGroup(group);
-    }
-    unsigned int vg = 0;
+    QStringList tabs;
     for (unsigned int i = 0, end = count(); i != end; ++i) {
-        if (const Page *const p = d->page(i)) {
+        if (Page *const p = d->page(i)) {
             if (p->isTemporary()) {
                 continue;
             }
-            KConfigGroup group(config, QString::asprintf("View #%u", vg++));
-            p->saveTo(group);
-            if (KCONFIG_DELETEGROUP_BROKEN) {
-                group.writeEntry("magic", 0xFA1AFE1U);
-            }
+            tabs += p->configGroup().name();
         }
     }
+    d->config->group(d->configKey).writeEntry("Tabs", tabs);
+    d->config->sync();
 }
 
 void TabWidget::connectSearchBar(SearchBar *sb)
